# This is a simulation study of the logistic-Weibull prevalence incidence mixture model when it is not misspecified

rm(list=ls(all=TRUE))
library(PIMixture)
#control number of simulations and samples
nsim <- 1000 #number of simulation data sets
n <- 10000  #number of samples in each data set
num <- nsim*n
b <- .1 #percent with baseline status known

#true parameter values
beta0 <- -3.5
beta1 <- 0.5
beta2 <- 1
gamma0 <- 3.5
gamma1 <- -0.5
gamma2 <- -0.5
tau <- 0.5

set.seed(1564)

#draw covariate to use for each simulation
x1 <- rbinom(n,1,0.5)  #covariate
x2 <- runif(n,0,1)  #covariate
p <- exp(beta0+beta1*x1+beta2*x2)/(1+exp(beta0+beta1*x1+beta2*x2))
lp <- exp(gamma0+gamma1*x1+gamma2*x2)

#calculate true risk
cin_0 <- p
cin_1 <- p+(1-p)*pweibull(1,1/tau,lp)
cin_3 <- p+(1-p)*pweibull(3,1/tau,lp)
cin_5 <- p+(1-p)*pweibull(5,1/tau,lp)
cin_10 <- p+(1-p)*pweibull(10,1/tau,lp)

#indicator function for status known at baseline
k <- rbinom(num,1,b)

set.seed(7456)

#draw time to event
c_tilde <- rbinom(num,1,p) #true c_i for all simulated datasets
t_tilde <- rweibull(num,1/tau,lp) #true t_i for all simulated datasets
t_tilde[which(c_tilde==1)] <- 0 #if c_i = 1, set t to 0

#draw time of 2nd to 10th visit
u2 <- rgamma(num,3,1)
u3 <- u2 + rgamma(num,3,1)
u4 <- u3 + rgamma(num,3,1)
u5 <- u4 + rgamma(num,3,1)
u6 <- u5 + rgamma(num,3,1)
u7 <- u6 + rgamma(num,3,1)
u8 <- u7 + rgamma(num,3,1)
u9 <- u8 + rgamma(num,3,1)
u10 <- u9 + rgamma(num,3,1)

#censoring distribution with a max at 20
nvis <- sample(seq(1,10,1),num,replace=TRUE)

u2 <- ifelse(nvis==1 | u2>20,Inf,u2)
u3 <- ifelse(nvis<=2 | u3>20,Inf,u3)
u4 <- ifelse(nvis<=3 | u4>20,Inf,u4)
u5 <- ifelse(nvis<=4 | u5>20,Inf,u5)
u6 <- ifelse(nvis<=5 | u6>20,Inf,u6)
u7 <- ifelse(nvis<=6 | u7>20,Inf,u7)
u8 <- ifelse(nvis<=7 | u8>20,Inf,u8)
u9 <- ifelse(nvis<=8 | u9>20,Inf,u9)
u10 <- ifelse(nvis<=9 | u10>20,Inf,u10)

#indicator variable for follow-up visit
k2 <- ifelse(I(u2==Inf)==0,1,0)

#set interval censored variables
ci <- rep(NA,num)

#if event is observed at baseline
ci[t_tilde==0 & k==1]  <- 1
ci[t_tilde>0 & k==1] <- 0

li <- ifelse(t_tilde > u2,
      ifelse(t_tilde > u3,
      ifelse(t_tilde > u4,
      ifelse(t_tilde > u5,
      ifelse(t_tilde > u6,
      ifelse(t_tilde > u7,
      ifelse(t_tilde > u8,
      ifelse(t_tilde > u9,
      ifelse(t_tilde > u10,u10,u9),u8),u7),u6),u5),u4),u3),u2),0)
      
ci[is.na(ci)==1 & li>0] <- 0      

ri <- ifelse(t_tilde <= u10,
      ifelse(t_tilde <= u9,       
      ifelse(t_tilde <= u8,
      ifelse(t_tilde <= u7,
      ifelse(t_tilde <= u6,
      ifelse(t_tilde <= u5,
      ifelse(t_tilde <= u4,
      ifelse(t_tilde <= u3,
      ifelse(t_tilde <= u2,
      ifelse(is.na(ci)==0 & ci==1,0,u2),u3),u4),u5),u6),u7),u8),u9),u10),Inf)      


#knowledge about CIN2+ at enrollment
k_obs <- k
k_obs[which(t_tilde>u2 & k2==1)] <- 1

table(k_obs,is.na(ci)) # %ci known
table(ci,I(ri==Inf),useNA="always")

p_1 <- exp(beta0+beta1+beta2*.5)/(1+exp(beta0+beta1+beta2*.5))
lp_1 <- exp(gamma0+gamma1+gamma2*.5)

truevalues <- data.frame(beta0, beta1, beta2, gamma0, gamma1, gamma2, tau, t(rep(NA,7)), 
                        p_1, 
                        p_1+(1-p_1)*pweibull(1,1/tau,lp_1),
                        p_1+(1-p_1)*pweibull(3,1/tau,lp_1),
                        p_1+(1-p_1)*pweibull(5,1/tau,lp_1),
                        p_1+(1-p_1)*pweibull(10,1/tau,lp_1),
                        t(rep(NA,10)),
                        t(rep(NA,29)))

colnames(truevalues)<- c("beta0", "beta1", "beta2", "gamma0", "gamma1", "gamma2", "tau", 
                         "beta0-95% coverage", "beta1-95% coverage", "beta2-95% coverage", "gamma0-95% coverage", "gamma1-95% coverage", "gamma2-95% coverage", "tau-95% coverage",
                         "CDF(t=0) for x1=1,x2=.5", "CDF(t=1) for x1=1,x2=.5", "CDF(t=3) for x1=1,x2=.5", "CDF(t=5) for x1=1,x2=.5", "CDF(t=10) for x1=1,x2=.5", 
                         "CDF(t=0) for x1=1,x2=.5--sd", "CDF(t=1) for x1=1,x2=.5--sd", "CDF(t=3) for x1=1,x2=.5--sd", "CDF(t=5) for x1=1,x2=.5--sd", "CDF(t=10) for x1=1,x2=.5--sd", 
                         "CDF(t=0) for x1=1,x2=.5--95% coverage", "CDF(t=1) for x1=1,x2=.5--95% coverage", "CDF(t=3) for x1=1,x2=.5--95% coverage",
                         "CDF(t=5) for x1=1,x2=.5--95% coverage", "CDF(t=10) for x1=1,x2=.5--95% coverage",
                         "sd(beta0)", "cov(beta0,beta1)", "cov(beta0,beta2)", "cov(beta0,gamma0)", "cov(beta0,gamma1)", "cov(beta0,gamma2)", "cov(beta0,tau)", 
                         "sd(beta1)", "cov(beta1,beta2)", "cov(beta1,gamma0)", "cov(beta1,gamma1)", "cov(beta1,gamma2)", "cov(beta1,tau)",
                         "sd(beta2)", "cov(beta2,gamma0)", "cov(beta2,gamma1)", "cov(beta2,gamma2)", "cov(beta2,tau)", 
                         "sd(gamma0)", "cov(gamma0,gamma1)", "cov(gamma0,gamma2)", "cov(gamma0,tau)", 
                         "sd(gamma1)", "cov(gamma1,gamma2)", "cov(gamma1,tau)", 
                         "sd(gamma2)", "cov(gamma2,tau)", 
                         "sd(tau)", "likelihood")

#the data
xsam_full <- cbind(rep(x1,nsim),rep(x2,nsim),k_obs,ci,li,ri,k2)
colnames(xsam_full) <- c("x1","x2","k","ci","li","ri","k2")

simsum <- data.frame()
for (j in 1:nsim){
   print(j)
   lower <- (j-1)*n+1
   upper <- j*n
   xsam <- data.frame(xsam_full[lower:upper,])
   
   model <- "ci+li+ri~x1+x2"
   res <- PIMixture(p.model=model, data=xsam, model="logistic-Weibull")
   fisher_info <- res$covariance
   times <- c(0,1,3,5,10)
   test.data <- data.frame(1,.5)
   names(test.data) <- c("x1","x2")
   pred <- PIMixture.predict(x=res,data=test.data,time.points=times)
   
      
   simres <- data.frame(t(res$regression.coef[,3]),
                        pmin(1*(res$regression.coef[,3]-1.96*sqrt(diag(fisher_info)) <= truevalues[1:7]),1*(truevalues[1:7] <= res$regression.coef[,3]+1.96*sqrt(diag(fisher_info)))),
                        t(pred$CR),t(pred$CR.se),
                        pmin(1*(pred$LL95 <= truevalues[15:19]),1*(truevalues[15:19] <= pred$UL95)),
                        sqrt(fisher_info[1,1]), fisher_info[1,2], fisher_info[1,3], fisher_info[1,4], fisher_info[1,5], fisher_info[1,6], fisher_info[1,7], 
                        sqrt(fisher_info[2,2]), fisher_info[2,3], fisher_info[2,4], fisher_info[2,5], fisher_info[2,6], fisher_info[2,7],
                        sqrt(fisher_info[3,3]), fisher_info[3,4], fisher_info[3,5], fisher_info[3,6], fisher_info[3,7], 
                        sqrt(fisher_info[4,4]), fisher_info[4,5], fisher_info[4,6], fisher_info[4,7],
                        sqrt(fisher_info[5,5]), fisher_info[5,6], fisher_info[5,7],
                        sqrt(fisher_info[6,6]), fisher_info[6,7],
                        sqrt(fisher_info[7,7]),
                        res$loglikelihood)

   colnames(simres)<- c("beta0", "beta1", "beta2", "gamma0", "gamma1", "gamma2", "tau", 
                        "beta0-95% coverage", "beta1-95% coverage", "beta2-95% coverage", "gamma0-95% coverage", "gamma1-95% coverage", "gamma2-95% coverage", "tau-95% coverage",
                        "CDF(t=0) for x1=1,x2=.5", "CDF(t=1) for x1=1,x2=.5", "CDF(t=3) for x1=1,x2=.5", "CDF(t=5) for x1=1,x2=.5", "CDF(t=10) for x1=1,x2=.5", 
                        "CDF(t=0) for x1=1,x2=.5--sd", "CDF(t=1) for x1=1,x2=.5--sd", "CDF(t=3) for x1=1,x2=.5--sd", "CDF(t=5) for x1=1,x2=.5--sd", "CDF(t=10) for x1=1,x2=.5--sd", 
                        "CDF(t=0) for x1=1,x2=.5--95% coverage", "CDF(t=1) for x1=1,x2=.5--95% coverage", "CDF(t=3) for x1=1,x2=.5--95% coverage",
                        "CDF(t=5) for x1=1,x2=.5--95% coverage", "CDF(t=10) for x1=1,x2=.5--95% coverage",
                        "sd(beta0)", "cov(beta0,beta1)", "cov(beta0,beta2)", "cov(beta0,gamma0)", "cov(beta0,gamma1)", "cov(beta0,gamma2)", "cov(beta0,tau)", 
                        "sd(beta1)", "cov(beta1,beta2)", "cov(beta1,gamma0)", "cov(beta1,gamma1)", "cov(beta1,gamma2)", "cov(beta1,tau)",
                        "sd(beta2)", "cov(beta2,gamma0)", "cov(beta2,gamma1)", "cov(beta2,gamma2)", "cov(beta2,tau)", 
                        "sd(gamma0)", "cov(gamma0,gamma1)", "cov(gamma0,gamma2)", "cov(gamma0,tau)", 
                        "sd(gamma1)", "cov(gamma1,gamma2)", "cov(gamma1,tau)", 
                        "sd(gamma2)", "cov(gamma2,tau)", 
                        "sd(tau)", "likelihood")
   
   simsum <- rbind(simsum,simres)                
}
                         
sumres <- rbind(truevalues,simsum,colMeans(simsum),apply(simsum,2,sd),100*(colMeans(simsum)-truevalues))
rownames(sumres)[1] <- "True values"
rownames(sumres)[2:(nsim+1)] <- rownames(simsum)
rownames(sumres)[(nsim+2)] <- "Mean"
rownames(sumres)[(nsim+3)] <- "SD"
rownames(sumres)[(nsim+4)] <- "% Bias"
                                                   
filename <- "simres_correctly_specified.csv"
#All simulation results with the first row as true values
write.table(sumres,file=paste(folder,filename,sep=""), sep=",", row.names=TRUE)