# This is a simulation study of the non-parametric cumulative risk estimator

rm(list=ls(all=TRUE))
library(PIMixture)

#control number of simulations and samples
nsim <- 1000 #number of simulation data sets - this will take a very long time to run
n <- 10000  #number of samples in each data set
num <- nsim*n
b <- .1 #percent with baseline status known (.1 = low disease ascertainment, .9 = high disease ascertainment)

#true parameter values
beta0 <- -2.5
gamma0 <- 3
tau <- 0.5

p <- exp(beta0)/(1+exp(beta0))
lp <- exp(gamma0)

#calculate true risk
cin_0 <- p
cin_1 <- p+(1-p)*pweibull(1,1/tau,lp)
cin_3 <- p+(1-p)*pweibull(3,1/tau,lp)
cin_5 <- p+(1-p)*pweibull(5,1/tau,lp)
cin_10 <- p+(1-p)*pweibull(10,1/tau,lp)

set.seed(1564)

#indicator function for status known at baseline
k <- rbinom(num,1,b)

set.seed(8945)

#step 2: draw time to event
c_tilde <- rbinom(num,1,p) #true c_i for all simulated datasets
t_tilde <- rweibull(num,1/tau,lp) #true t_i for all simulated datasets
t_tilde[which(c_tilde==1)] <- 0 #if c_i = 1, set t to 0

#draw time of 2nd to 10th visit
u2 <- sample(1:9, size=num,replace=TRUE,prob=c(.09,.25,.28,.18,.12,.07,.04,.02,.01))
u3 <- u2 + sample(1:9, size=num,replace=TRUE,prob=c(.09,.25,.28,.18,.12,.07,.04,.02,.01))
u4 <- u3 + sample(1:9, size=num,replace=TRUE,prob=c(.09,.25,.28,.18,.12,.07,.04,.02,.01))
u5 <- u4 + sample(1:9, size=num,replace=TRUE,prob=c(.09,.25,.28,.18,.12,.07,.04,.02,.01))
u6 <- u5 + sample(1:9, size=num,replace=TRUE,prob=c(.09,.25,.28,.18,.12,.07,.04,.02,.01))
u7 <- u6 + sample(1:9, size=num,replace=TRUE,prob=c(.09,.25,.28,.18,.12,.07,.04,.02,.01))
u8 <- u7 + sample(1:9, size=num,replace=TRUE,prob=c(.09,.25,.28,.18,.12,.07,.04,.02,.01))
u9 <- u8 + sample(1:9, size=num,replace=TRUE,prob=c(.09,.25,.28,.18,.12,.07,.04,.02,.01))
u10 <- u9 + sample(1:9, size=num,replace=TRUE,prob=c(.09,.25,.28,.18,.12,.07,.04,.02,.01))

#censoring distribution with a max at 20
nvis <- sample(seq(1,10,1),num,replace=TRUE)

u2 <- ifelse(nvis==1 | u2>20,Inf,u2)
u3 <- ifelse(nvis<=2 | u3>20,Inf,u3)
u4 <- ifelse(nvis<=3 | u4>20,Inf,u4)
u5 <- ifelse(nvis<=4 | u5>20,Inf,u5)
u6 <- ifelse(nvis<=5 | u6>20,Inf,u6)
u7 <- ifelse(nvis<=6 | u7>20,Inf,u7)
u8 <- ifelse(nvis<=7 | u8>20,Inf,u8)
u9 <- ifelse(nvis<=8 | u9>20,Inf,u9)
u10 <- ifelse(nvis<=9 | u10>20,Inf,u10)

#indicator variable for follow-up visit
k2 <- ifelse(I(u2==Inf)==0,1,0)

#set interval censored variables
ci <- rep(NA,num)

#if event is observed at baseline
ci[t_tilde==0 & k==1]  <- 1
ci[t_tilde>0 & k==1] <- 0

li <- ifelse(t_tilde > u2,
      ifelse(t_tilde > u3,
      ifelse(t_tilde > u4,
      ifelse(t_tilde > u5,
      ifelse(t_tilde > u6,
      ifelse(t_tilde > u7,
      ifelse(t_tilde > u8,
      ifelse(t_tilde > u9,
      ifelse(t_tilde > u10,u10,u9),u8),u7),u6),u5),u4),u3),u2),0)
      
ci[is.na(ci)==1 & li>0] <- 0      

ri <- ifelse(t_tilde <= u10,
      ifelse(t_tilde <= u9,       
      ifelse(t_tilde <= u8,
      ifelse(t_tilde <= u7,
      ifelse(t_tilde <= u6,
      ifelse(t_tilde <= u5,
      ifelse(t_tilde <= u4,
      ifelse(t_tilde <= u3,
      ifelse(t_tilde <= u2,
      ifelse(is.na(ci)==0 & ci==1,0,u2),u3),u4),u5),u6),u7),u8),u9),u10),Inf)      

#knowledge about CIN2+ at enrollment
k_obs <- k
k_obs[which(t_tilde>u2 & k2==1)] <- 1

table(k_obs,is.na(ci)) # %ci known
table(ci,I(ri==Inf),useNA="always")

truevalues <- data.frame(cin_0, cin_1, cin_3, cin_5, cin_10, t(rep(NA,5)))

colnames(truevalues)<- c("CDF(t=0)","CDF(t=1)","CDF(t=3)","CDF(t=5)","CDF(t=10)",
                         "CDF(t=0)--95% coverage","CDF(t=1)--95% coverage","CDF(t=3)--95% coverage","CDF(t=5)--95% coverage","CDF(t=10)--95% coverage")

#the data
xsam_full <- cbind(k_obs,ci,li,ri,k2)
colnames(xsam_full) <- c("k","ci","li","ri","k2")

simsum <- data.frame()
for (j in 1:nsim){
   print(j)
   lower <- (j-1)*n+1
   upper <- j*n
   xsam <- data.frame(xsam_full[lower:upper,])

   model <- "ci+li+ri~1"   
   res <- PIMixture(p.model=model, data=xsam, model="non-parametric", conf.int = TRUE)
   times <- c(0,1,3,5,10)
   pred <- PIMixture.predict(res,time.points=times)
   
   simres <- data.frame(t(pred[,2]),pmin(1*(pred[,3] <= truevalues[1:5]),1*(truevalues[1:5] <= pred[,4])))
   colnames(simres)<- c("CDF(t=0)","CDF(t=1)","CDF(t=3)","CDF(t=5)","CDF(t=10)",
                         "CDF(t=0)--95% coverage","CDF(t=1)--95% coverage","CDF(t=3)--95% coverage","CDF(t=5)--95% coverage","CDF(t=10)--95% coverage")
   simsum <- rbind(simsum,simres)   
}

sumres <- rbind(truevalues,simsum,colMeans(simsum),apply(simsum,2,sd),100*(colMeans(simsum)-truevalues))
rownames(sumres)[1] <- "True values"
rownames(sumres)[2:(nsim+1)] <- rownames(simsum)
rownames(sumres)[(nsim+2)] <- "Mean"
rownames(sumres)[(nsim+3)] <- "SD"
rownames(sumres)[(nsim+4)] <- "% Bias"

filename <- "simres_nonparametric.csv"
#All simulation results with the first row as true values
write.table(sumres,file=paste(folder,filename,sep=""), sep=",", row.names=TRUE)
